clear all
close all
clc

load data_hospital.mat

%% Data
hospital_ID = data_hospital(:,1);   %... hospital ID
year        = data_hospital(:,2);   %... year
costs       = data_hospital(:,3);   %... log of hospital operating costs (in thousands of dollars)
beds        = data_hospital(:,4);   %... log of number of beds in hospital
inpatient   = data_hospital(:,5);   %... log of number of inpatients visit
outpatient  = data_hospital(:,6);   %... log of number of outpatient visit
case_mix    = data_hospital(:,7);   %... log of case mix index (higher values = difficult cases)
K           = data_hospital(:,8);   %... log of capital stock (in thousand dollars)
nonprofit   = data_hospital(:,9);   %... 1 for non-profit hospitals (=0 otherwise)
forprofit    = data_hospital(:,10); %... 1 for for-profit hospitals (=0 otherwise)


%% *********************************************

N = 382;  %pocet nemocnic
T = 5; %pocet let
k = 8;  %pocet parametru

%% Tvorba datovych matic

y = costs; %logaritmus nakladu
yi = reshape(y,T,N); %y_i pro i=1...N (po sloupcich)

X = [ones(N*T,1) beds inpatient outpatient case_mix K nonprofit forprofit];
Xi = zeros(T,k,N); %X_i pro i=1...N (po treti dimenzi)
for i = 1:N
   Xi(:,:,i) = X((T*i-T)+1:T*i,:); 
end
Xi_til = Xi(:,2:end,:); %X_i s vlnovkou (bez urovnove konstanty)

Xast = zeros(T*N,(N+k-1)); 
for i = 1:N
   Xast((T*i-T)+1:T*i,i)=ones(T,1);
   Xast((T*i-T)+1:T*i,end-(k-1)+1:end)=Xi_til(:,:,i);
end

tic

%% Pooled model
%"standardni" Gibbsuv vzorkovac - jen vetsi matice a vektory
%prior hyperparameters
    b0 = [1 1 1 1 1 1 1 1]';

    s02 = 0.5^2;
    h0 = 1/s02;
    varb0 = [1^2 1^2 1^2 1^2 1^2 1^2 1^2 1^2]';
    nu0 = 1;
    varh0 = 2*h0^2/nu0;
    V0 = diag(varb0);

    S = 10000;
    S0 = 1000;
    S1 = S-S0;

b_Gibbs = zeros(k,S);
h_Gibbs = zeros(1,S);

%Gibbsuv vzorkovac - 1. beh
h_Gibbs0 = h0;
V1 = inv(inv(V0)+h_Gibbs0*X'*X);
b1 = V1*(inv(V0)*b0+h_Gibbs0*X'*y);
b_Gibbs(:,1)=b1+norm_rnd(V1); %podminena hustota pro beta

nu1 =T*N+nu0;
s12 = 1/nu1*((y-X*b_Gibbs(:,1))'*(y-X*b_Gibbs(:,1))+nu0*s02);
h_Gibbs(:,1)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h

fprintf('Posteriorni simulace\n');
fprintf('0%%       50%%      100%% \n');
fprintf('|');
for i=2:S

   V1 = inv(inv(V0)+h_Gibbs(1,i-1)*X'*X);
   b1 = V1*(inv(V0)*b0+h_Gibbs(1,i-1)*X'*y);
   b_Gibbs(:,i)=b1+norm_rnd(V1);  %podminena hustota pro beta
   s12 = 1/nu1*((y-X*b_Gibbs(:,i))'*(y-X*b_Gibbs(:,i))+nu0*s02);
   h_Gibbs(:,i)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h
 
    if mod(i/S*100,5) == 0;    
      fprintf('|');
    end
end

fprintf('\n');
fprintf('Hotovo!\n');


%vyhozeni prvnich S0 vzorku
    b_S1 = b_Gibbs(:,S0+1:S);
    h_S1 = h_Gibbs(:,S0+1:S);

    b_mean = mean(b_S1,2); %posteriorni stredni hodnota 
    h_mean = mean(h_S1);
    b_var = mean(b_S1.^2,2)-b_mean.^2; %posteriorni rozptyl
    h_var = mean(h_S1.^2)-h_mean.^2;

%KONVERGENCNI DIAGNOSTIKY - pro uspornost neresim 
tt = toc;

%Prezentace vysledku, grafy
fprintf('Pooled model\n');
fprintf('Prior and posterior means for beta (std. deviations in parentheses)\n');
fprintf('                Prior       Posterior      \n');
fprintf('                                           \n');
fprintf('   ------------------------------------\n');

for i=1:k
    fprintf('Beta%1.0f     %12.4f   %12.4f   \n',[i b0(i) b_mean(i)]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varb0(i)) sqrt(b_var(i))]);
end

    fprintf('h         %12.4f   %12.4f   \n',[h0 h_mean]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varh0) sqrt(h_var)]);


fprintf('   ------------------------------------\n');
fprintf('Number of replications: %12.4f total \n',S);
fprintf('                        %12.4f discarded \n',S0);
fprintf('   ------------------------------------\n');
fprintf('Time elapsed (s): %12.2f \n',tt);
fprintf('***************************************************************************\n\n\n');











%% Individual effects model - nehierarchicka apriorni hustota
%prior hyperparameters
    a0 = ones(N,1)';       %variabilni urovnove konstanty napric aerolinkami
    b0 = [a0 1 1 1 1 1 1 1]';     %stejne ostatni parametry (sklonu)

    s02 = 1^2;
    h0 = 1/s02;
    
    vara0 = ones(N,1)';
    varb0 = [vara0 1^2 1^2 1^2 1^2 1^2 1^2 1^2]';
    nu0 = 1;
    varh0 = 2*h0^2/nu0;
    V0 = diag(varb0);

    S = 1000;
    S0 = 500;
    S1 = S-S0;

b_Gibbs = zeros(N+k-1,S);
h_Gibbs = zeros(1,S);

%Gibbsuv vzorkovac - 1. beh
h_Gibbs0 = h0;
V1 = inv(inv(V0)+h_Gibbs0*Xast'*Xast);
b1 = V1*(inv(V0)*b0+h_Gibbs0*Xast'*y);
b_Gibbs(:,1)=b1+norm_rnd(V1); %podminena hustota pro beta

nu1 =T*N+nu0;
   pom = 0;
   for j=1:N
      pom = pom+(yi(:,j)-b_Gibbs(j,1)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(N+1:N+(k-1),1))'*(yi(:,j)-b_Gibbs(j,1)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(N+1:N+(k-1),1));
   end
   s12 = 1/nu1*(pom+nu0*s02);
h_Gibbs(:,1)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h

fprintf('Posteriorni simulace\n');
fprintf('0%%       50%%      100%% \n');
fprintf('|');
for i=2:S
    
   V1 = inv(inv(V0)+h_Gibbs(1,i-1)*Xast'*Xast);
   b1 = V1*(inv(V0)*b0+h_Gibbs(1,i-1)*Xast'*y);
   b_Gibbs(:,i)=b1+norm_rnd(V1);  %podminena hustota pro beta
   pom = 0;
   for j=1:N
      pom = pom+(yi(:,j)-b_Gibbs(j,i)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(N+1:N+(k-1),i))'*(yi(:,j)-b_Gibbs(j,i)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(N+1:N+(k-1),i));
   end
   s12 = 1/nu1*(pom+nu0*s02);
   h_Gibbs(:,i)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h
   
   if mod(i/S*100,5) == 0;    
      fprintf('|');
   end
end

fprintf('\n');
fprintf('Hotovo!\n');

%vyhozeni prvnich S0 vzorku
    b_S1 = b_Gibbs(:,S0+1:S);
    h_S1 = h_Gibbs(:,S0+1:S);

    b_mean = mean(b_S1,2); %posteriorni stredni hodnota 
    h_mean = mean(h_S1);
    b_var = mean(b_S1.^2,2)-b_mean.^2; %posteriorni rozptyl
    h_var = mean(h_S1.^2)-h_mean.^2;

%KONVERGENCNI DIAGNOSTIKY - pro uspornost neresim 
tt = toc;

%Prezentace vysledku, grafy
fprintf('Individual Effects Model - nehierarchicka apriorni hustota\n');
fprintf('Prior and posterior means for beta (std. deviations in parentheses)\n');
fprintf('                Prior       Posterior      \n');
fprintf('                                           \n');
fprintf('   ------------------------------------\n');

for i=1:N
    fprintf('Alfa%1.0f     %12.4f   %12.4f   \n',[i b0(i) b_mean(i)]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varb0(i)) sqrt(b_var(i))]);
end
for i=N+1:N+(k-1)
    fprintf('Beta%1.0f     %12.4f   %12.4f   \n',[i-N+1 b0(i) b_mean(i)]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varb0(i)) sqrt(b_var(i))]);
end

    fprintf('h         %12.4f   %12.4f   \n',[h0 h_mean]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varh0) sqrt(h_var)]);


fprintf('   ------------------------------------\n');
fprintf('Number of replications: %12.4f total \n',S);
fprintf('                        %12.4f discarded \n',S0);
fprintf('   ------------------------------------\n');
fprintf('Time elapsed (s): %12.2f \n',tt);
fprintf('***************************************************************************\n\n\n');












%% Individual effects model - hierarchicka apriorni hustota
%prior hyperparameters
    a0 = ones(N,1)';       %variabilni urovnove konstanty napric aerolinkami
    b0 = [1 1 1 1 1 1 1]';     %stejne ostatni parametry (sklonu)

    s02 = 1^2;
    h0 = 1/s02;
    
    vara0 = ones(N,1)';
    varb0 = [1^2 1^2 1^2 1^2 1^2 1^2 1^2]';
    nu0 = 1;
    varh0 = 2*h0^2/nu0;
    V0 = diag(varb0);

    mu0_a = 5;
    s02_a = 2^2;
    V0_a = 5;
    nu0_a = 1;
    
    
    S = 1000;
    S0 = 500;
    S1 = S-S0;

b_Gibbs = zeros(k-1,S);
h_Gibbs = zeros(1,S);
a_Gibbs = zeros(N,S);

V_a_Gibbs = zeros(1,S);
mu_a_Gibbs = zeros(1,S);

%Gibbsuv vzorkovac - 1. beh
h_Gibbs0 = h0;
V_a_Gibbs0 = V0_a;
mu_a_Gibbs0 = mu0_a;
a_Gibbs0 = a0;

pom = 0;
   for j=1:N
      pom = pom+Xi_til(:,:,j)'*Xi_til(:,:,j);
   end
V1 = inv(inv(V0)+h_Gibbs0*pom);

pom = 0;
   for j=1:N
      pom = pom+Xi_til(:,:,j)'*(yi(:,j)-a_Gibbs0(j)*ones(T,1));
   end
b1 = V1*(inv(V0)*b0+h_Gibbs0*pom);
b_Gibbs(:,1)=b1+norm_rnd(V1); %podminena hustota pro beta

nu1 =T*N+nu0;
   pom = 0;
   for j=1:N
      pom = pom+(yi(:,j)-a_Gibbs0(j)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(:,1))'*(yi(:,j)-a_Gibbs0(j)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(:,1));
   end
   s12 = 1/nu1*(pom+nu0*s02);
h_Gibbs(:,1)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h

%podminene hustoty pro kazde alfa_i
for j=1:N
   V1_i = (V_a_Gibbs0*h_Gibbs(:,1)^-1)/(T*V_a_Gibbs0+h_Gibbs(:,1)^-1);
   a1 = (V_a_Gibbs0*(yi(:,j)-Xi_til(:,:,j)*b_Gibbs(:,1))'*ones(T,1)+h_Gibbs(:,1)^-1*mu_a_Gibbs0)/(T*V_a_Gibbs0+h_Gibbs(:,1)^-1);
   a_Gibbs(j,1) = a1+norm_rnd(V1_i);  %Koop 7.16 
end

%podminena hustota pro mu_alfa
s12_a = V_a_Gibbs0*s02_a/(V_a_Gibbs0+N*s02_a);
mu1_a = (V_a_Gibbs0*mu0_a+s02_a*sum(a_Gibbs(:,1)))/(V_a_Gibbs0+N*s02_a);
mu_a_Gibbs(:,1) = mu1_a+norm_rnd(s12_a); %Koop 7.17


%podminena hustota pro inv(V_alfa)
nu1_a =nu0_a+N;
   pom = 0;
   for j=1:N
      pom = pom+(a_Gibbs(j,1)-mu_a_Gibbs(:,1))^2;
   end
V1_a = (pom+V0_a*nu0_a)/nu1_a;
V_a_Gibbs(:,1)=inv(gamm_rnd(1,1,.5*nu1_a,.5*nu1_a*inv(V1_a))); %Koop 7.18




%Dalsi beh Gibbsova vzorkovace
fprintf('Posteriorni simulace\n');
fprintf('0%%       50%%      100%% \n');
fprintf('|');
for i=2:S

pom = 0;
   for j=1:N
      pom = pom+Xi_til(:,:,j)'*Xi_til(:,:,j);
   end
V1 = inv(inv(V0)+h_Gibbs(:,i-1)*pom);

pom = 0;
   for j=1:N
      pom = pom+Xi_til(:,:,j)'*(yi(:,j)-a_Gibbs(j,i-1)*ones(T,1));
   end
b1 = V1*(inv(V0)*b0+h_Gibbs(:,i-1)*pom);
b_Gibbs(:,i)=b1+norm_rnd(V1); %podminena hustota pro beta

nu1 =T*N+nu0;
   pom = 0;
   for j=1:N
      pom = pom+(yi(:,j)-a_Gibbs(j,i-1)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(:,i))'*(yi(:,j)-a_Gibbs(j,i-1)*ones(T,1)-Xi_til(:,:,j)*b_Gibbs(:,i));
   end
   s12 = 1/nu1*(pom+nu0*s02);
h_Gibbs(:,i)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h

%podminene hustoty pro kazde alfa_i
for j=1:N
   V1_i = (V_a_Gibbs(:,i-1)*h_Gibbs(:,i)^-1)/(T*V_a_Gibbs(:,i-1)+h_Gibbs(:,i)^-1);
   a1 = (V_a_Gibbs(:,i-1)*(yi(:,j)-Xi_til(:,:,j)*b_Gibbs(:,i))'*ones(T,1)+h_Gibbs(:,i)^-1*mu_a_Gibbs(:,i-1))/(T*V_a_Gibbs(:,i-1)+h_Gibbs(:,i)^-1);
   a_Gibbs(j,i) = a1+norm_rnd(V1_i);  %Koop 7.16 
end

%podminena hustota pro mu_alfa
s12_a = V_a_Gibbs(:,i-1)*s02_a/(V_a_Gibbs(:,i-1)+N*s02_a);
mu1_a = (V_a_Gibbs(:,i-1)*mu0_a+s02_a*sum(a_Gibbs(:,i)))/(V_a_Gibbs(:,i-1)+N*s02_a);
mu_a_Gibbs(:,i) = mu1_a+norm_rnd(s12_a); %Koop 7.17


%podminena hustota pro inv(V_alfa)
nu1_a =nu0_a+N;
   pom = 0;
   for j=1:N
      pom = pom+(a_Gibbs(j,i)-mu_a_Gibbs(:,i))^2;
   end
V1_a = (pom+V0_a*nu0_a)/nu1_a;
V_a_Gibbs(:,i)=inv(gamm_rnd(1,1,.5*nu1_a,.5*nu1_a*inv(V1_a))); %Koop 7.18


    if mod(i/S*100,5) == 0;    
      fprintf('|');
    end
end

fprintf('\n');
fprintf('Hotovo!\n');



%vyhozeni prvnich S0 vzorku
    a_S1 = a_Gibbs(:,S0+1:S);
    b_S1 = b_Gibbs(:,S0+1:S);
    h_S1 = h_Gibbs(:,S0+1:S);

    b_mean = mean(b_S1,2); %posteriorni stredni hodnota 
    a_mean = mean(a_S1,2);
    h_mean = mean(h_S1);
    b_var = mean(b_S1.^2,2)-b_mean.^2; %posteriorni rozptyl
    a_var = mean(a_S1.^2,2)-a_mean.^2;
    h_var = mean(h_S1.^2)-h_mean.^2;

%KONVERGENCNI DIAGNOSTIKY - pro uspornost neresim 
tt = toc;

%Prezentace vysledku, grafy
fprintf('Individual Effects Model - hierarchicka apriorni hustota\n');
fprintf('Prior and posterior means for beta (std. deviations in parentheses)\n');
fprintf('                Prior       Posterior      \n');
fprintf('                                           \n');
fprintf('   ------------------------------------\n');

for i=1:N
    fprintf('Alfa%1.0f     %12.4f   %12.4f   \n',[i a0(i) a_mean(i)]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(vara0(i)) sqrt(a_var(i))]);
end
for i=1:(k-1)
    fprintf('Beta%1.0f     %12.4f   %12.4f   \n',[i+1 b0(i) b_mean(i)]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varb0(i)) sqrt(b_var(i))]);
end

    fprintf('h         %12.4f   %12.4f   \n',[h0 h_mean]);
    fprintf('           (%12.4f) (%12.4f)\n',[sqrt(varh0) sqrt(h_var)]);


fprintf('   ------------------------------------\n');
fprintf('Number of replications: %12.4f total \n',S);
fprintf('                        %12.4f discarded \n',S0);
fprintf('   ------------------------------------\n');
fprintf('Time elapsed (s): %12.2f \n',tt);
fprintf('***************************************************************************\n\n\n');













% %% Random coefficients model - hierarchicka apriorni hustota
% %prior hyperparameters
%  b0 = zeros(k,N);
%  for j=1:N
%     b0(:,j) = [1 1 1 1 1 1 1 1]';     %vsechny parametry variabilni
%  end
%     s02 = 1^2;
%     h0 = 1/s02;
%     
%     varb0 = [1^2 1^2 1^2 1^2 1^2 1^2 1^2 1^2]';
%     nu0 = 1;
%     varh0 = 2*h0^2/nu0;
%     V0 = diag(varb0);
% 
%     mu0_b = [1 1 1 1 1 1 1 1]';
%     Sigma0_b = diag(varb0);
%     V0_b = V0;
%     nu0_b = 5;
%     
%     
%     S = 1000;
%     S0 = 500;
%     S1 = S-S0;
% 
% b_Gibbs = zeros(k,S,N);
% h_Gibbs = zeros(1,S);
% 
% V_b_Gibbs = zeros(k,k,S);
% mu_b_Gibbs = zeros(k,S);
% 
% %Gibbsuv vzorkovac - 1. beh
% h_Gibbs0 = h0;
% V_b_Gibbs0 = V0_b;
% mu_b_Gibbs0 = mu0_b;
% 
%  for j=1:N
%     V1 = inv(h_Gibbs0*Xi(:,:,j)'*Xi(:,:,j)+inv(V_b_Gibbs0));
%     b1 = V1*(h_Gibbs0*Xi(:,:,j)'*yi(:,j)+inv(V_b_Gibbs0)*mu_b_Gibbs0);
%     b_Gibbs(:,1,j)=b1+norm_rnd(V1); %podminena hustota pro beta - Koop 7.25
%  end
%  
% %podminena hustota pro mu_beta - Koop (7.26) a naseldujici
% Sigma1_b = inv(N*inv(V_b_Gibbs0)+inv(Sigma0_b));
%    pom = 0;
%    for j=1:N
%       pom = pom+b_Gibbs(:,1,j);
%    end
% mu1_b = Sigma1_b*(inv(V_b_Gibbs0)*pom+inv(Sigma0_b)*mu0_b);
% mu_b_Gibbs(:,1) = mu1_b+norm_rnd(Sigma1_b);
% 
% nu1_b = N+nu0_b;
%    pom = 0;
%    for j=1:N
%       pom = pom+(b_Gibbs(:,1,j)-mu_b_Gibbs(:,1))*(b_Gibbs(:,1,j)-mu_b_Gibbs(:,1))';
%    end
% V1_b = pom+V0_b;
% V_b_Gibbs(:,:,1) = inv(wish_rnd(inv(nu1_b*V1_b),nu1_b));
% 
% %podminena hustota pro h
% nu1 = T*N+nu0;
%    pom = 0;
%    for j=1:N
%       pom = pom+(yi(:,j)-Xi(:,:,j)*b_Gibbs(:,1,j))'*(yi(:,j)-Xi(:,:,j)*b_Gibbs(:,1,j));
%    end
%    s12 = 1/nu1*(pom+nu0*s02);
% h_Gibbs(:,1)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h
% 
% 
% 
% %Dalsi beh Gibbsova vzorkovace
% for i=2:S
%  for j=1:N
%     V1 = inv(h_Gibbs(:,i-1)*Xi(:,:,j)'*Xi(:,:,j)+inv(V_b_Gibbs(:,:,i-1)));
%     b1 = V1*(h_Gibbs(:,i-1)*Xi(:,:,j)'*yi(:,j)+inv(V_b_Gibbs(:,:,i-1))*mu_b_Gibbs(:,i-1));
%     
%      %(nesystematicky zasah -- nekdy nastava numericky problem, ze Matlab nevezme V1 jako pozitivne
%      %definitni matici => ignorujeme chybu a "udelame z V1 pozitivne definitni
%      %prictenim maleho cisla 
%  
%     %try
%     b_Gibbs(:,i,j)=b1+norm_rnd(V1); %podminena hustota pro beta - Koop 7.25
%     %catch
%     %b_Gibbs(:,i,j)=b1+norm_rnd(V1+0.01);
%     %end
%  end
%  
% %podminena hustota pro mu_beta - Koop (7.26) a naseldujici
% Sigma1_b = inv(N*inv(V_b_Gibbs(:,:,i-1))+inv(Sigma0_b));
%    pom = 0;
%    for j=1:N
%       pom = pom+b_Gibbs(:,i,j);
%    end
% mu1_b = Sigma1_b*(inv(V_b_Gibbs(:,:,i-1))*pom+inv(Sigma0_b)*mu0_b);
% mu_b_Gibbs(:,i) = mu1_b+norm_rnd(Sigma1_b);
% 
% nu1_b = N+nu0_b;
%    pom = 0;
%    for j=1:N
%       pom = pom+(b_Gibbs(:,i,j)-mu_b_Gibbs(:,i))*(b_Gibbs(:,i,j)-mu_b_Gibbs(:,i))';
%    end
% V1_b = pom+V0_b;
% V_b_Gibbs(:,:,i) = inv(wish_rnd(inv(nu1_b*V1_b),nu1_b));
% 
% %podminena hustota pro h
% nu1 = T*N+nu0;
%    pom = 0;
%    for j=1:N
%       pom = pom+(yi(:,j)-Xi(:,:,j)*b_Gibbs(:,i,j))'*(yi(:,j)-Xi(:,:,j)*b_Gibbs(:,i,j));
%    end
%    s12 = 1/nu1*(pom+nu0*s02);
% h_Gibbs(:,i)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12); %podminena hustota pro h
% end
% 
% 
% 
% %vyhozeni prvnich S0 vzorku
%     b_S1 = b_Gibbs(:,S0+1:S,:);
%     h_S1 = h_Gibbs(:,S0+1:S);
%     mu_b_S1 = mu_b_Gibbs(:,S0+1:S);
% 
%     b_mean = zeros(k,N);
%     for j = 1:N
%     b_mean(:,j) = mean(b_S1(:,:,j),2); %posteriorni stredni hodnota 
%     end
%     h_mean = mean(h_S1);
%     mu_b_mean = mean(mu_b_S1,2);
% 
%     b_var = zeros(k,N);
%     for j = 1:N
%     b_var(:,j) = mean(b_S1(:,:,j).^2,2)-b_mean(:,j).^2; %posteriorni rozptyl 
%     end
%     h_var = mean(h_S1.^2)-h_mean.^2;
%     mu_b_var = mean(mu_b_S1.^2,2)-mu_b_mean.^2;
% 
%     
% %KONVERGENCNI DIAGNOSTIKY - pro uspornost neresim 
% tt = toc;
% 
% %Prezentace vysledku, grafy
% fprintf('Random Coefficients Model - hierarchicka apriorni hustota\n');
% fprintf('Posterior means for beta (std. deviations in parentheses)\n');
% fprintf('   Firma     1            2            3            4            5            6\n');
% fprintf('   -----------------------------------------------------------------------------------\n');
% 
% for i=1:k
%     fprintf('beta%1.0f   %10.4f   %10.4f   %10.4f   %10.4f   %10.4f   %10.4f \n',[i b_mean(i,:)]);
%     fprintf('           (%10.4f) (%10.4f) (%10.4f) (%10.4f) (%10.4f) (%10.4f)\n',[sqrt(b_var(i,:))]);
% end
% 
%     fprintf('h       %10.4f   %10.4f   \n',[h0 h_mean]);
%     fprintf('           (%10.4f) (%10.4f)\n',[sqrt(varh0) sqrt(h_var)]);
% 
% 
% fprintf('   ------------------------------------\n');
% fprintf('Number of replications: %12.4f total \n',S);
% fprintf('                        %12.4f discarded \n',S0);
% fprintf('   ------------------------------------\n');
% fprintf('Time elapsed (s): %12.2f \n',tt);
% fprintf('***************************************************************************\n\n\n');
% 
% 
% 
% 
